# SPDX-License-Identifier: Apache-2.0

__doc__ = """
Classes and functions that implement the ElasticsearchRetriever.
"""


class ElasticsearchRetriever:
    """Retriever for documents hosted on an ElasticSearch server."""

    def __init__(
        self,
        corpus_name: str,
        host: str,
        **kwargs: dict[str, int],
    ):
        """
        :param hosts: Full url:port to the Elasticsearch server.
        :param kwargs: Additional kwargs to pass to the Elasticsearch class.
        """

        # Third Party
        from elasticsearch import Elasticsearch

        self.corpus_name = corpus_name

        # Hosts is the minimum required param to init a connection to the
        # Elasticsearch server, so make it explicit here.
        self.hosts = host
        self.kwargs = kwargs

        self.es = Elasticsearch(hosts=host, **kwargs)

    def create_es_body(self, limit, query):
        """
        :param limit: Max number of documents to retrieve.
        :param query: Query string for retrieving documents.
        """

        body = {
            "size": limit,
            "query": {
                "bool": {
                    "must": {
                        "text_expansion": {
                            "ml.tokens": {
                                "model_id": ".elser_model_1",
                                "model_text": query,
                            }
                        }
                    }
                }
            },
        }
        return body

    def retrieve(self, query: str, top_k: int = 5) -> list[dict]:
        body = self.create_es_body(top_k, query)

        retriever_results = self.es.search(
            index=self.corpus_name,
            body=body,
        )
        hits = retriever_results["hits"]["hits"]

        # Format for the processor.
        documents = []
        for hit in hits:
            document = {
                "doc_id": hit["_id"],
                "text": hit["_source"]["text"],
                "score": str(hit["_score"]),  # Non-string values crash vLLM
            }
            documents.append(document)

        return documents



# SPDX-License-Identifier: Apache-2.0


# First Party
from granite_common.base.io import (
    Retriever,
)
from granite_common.retrievers import util
from granite_common.retrievers.elasticsearch import (
    ElasticsearchRetriever,
)
from granite_common.retrievers.embeddings import (
    InMemoryRetriever,
    compute_embeddings,
    write_embeddings,
)

# Expose public symbols at `granite_common.io.retrievers` to save users from typing
__all__ = [
    "Retriever",
    "InMemoryRetriever",
    "ElasticsearchRetriever",
    "compute_embeddings",
    "write_embeddings",
    "util",
]



# SPDX-License-Identifier: Apache-2.0

"""
Utilities for optional dependencies
"""

# Standard
from contextlib import contextmanager
import logging

_NLTK_INSTALL_INSTRUCTIONS = """
Please install nltk with:
    pip install nltk
In some environments you may also need to manually download model weights with:
    python -m nltk.downloader punkt_tab
See https://www.nltk.org/install.html#installing-nltk-data for more detailed 
instructions."""


@contextmanager
def import_optional(extra_name: str):
    """Context manager to handle optional imports"""
    try:
        yield
    except ImportError as err:
        logging.warning(
            "%s.\nHINT: You may need to pip install %s[%s]",
            err,
            __package__,
            extra_name,
        )
        raise


@contextmanager
def nltk_check(feature_name: str):
    """Variation on import_optional for nltk.

    :param feature_name: Name of feature that requires NLTK"""
    try:
        yield
    except ImportError as err:
        raise ImportError(
            f"'nltk' package not installed. This package is required for "
            f"{feature_name} in the 'granite_io' library."
            f"{_NLTK_INSTALL_INSTRUCTIONS}"
        ) from err
